"""Server-wide constants and configuration defaults."""

# Default model to use when no model is configured
# This is used as a fallback in cli.py and shown as an example in api_v2_sessions.py
DEFAULT_FALLBACK_MODEL = "anthropic/claude-sonnet-4-6"
